@@ -7,7 +7,8 @@
 ;; Keywords: tools, accessibility
 ;; URL: https://gitlab.com/michelangelo-rodriguez/greader
 
-;; Version: 0.13.1
+;; Package-Version: 20251019.1127
+;; Package-Revision: 3865fe9b9439
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -44,11 +45,6 @@
 ;;; Code:
 (require 'seq)
 (require 'view)
-(defvar greader-dict-prefix-map)
-(defvar greader-dict-mode)
-(defvar greader-dict-toggle-filters)
-(declare-function greader-dict--update nil)
-
 (defvar-local greader-timer-flag nil)
 (require 'find-func)
 (defvar greader-auto-tired-timer nil)
@@ -257,27 +253,16 @@ if set to t, when you call function `greader-read', that function sets a
   (setq greader-reading-mode nil))
 
 (define-obsolete-variable-alias 'greader-map 'greader-mode-map "2022")
-
-(defcustom greader-keymap-prefix "C-r"
-  "The prefix for `greader-mode' commands."
-  :type 'string
-  :group 'greader)
-
-(defvar greader-prefix-keymap
-  (let ((map (make-sparse-keymap)))
-    (define-key map (kbd "s")   #'greader-toggle-tired-mode)
-    (define-key map (kbd "r")   #'isearch-backward)
-    (define-key map (kbd "SPC") #'greader-read)
-    (define-key map (kbd "l")   #'greader-set-language)
-    (define-key map (kbd "t")   #'greader-toggle-timer)
-    (define-key map (kbd "f")   #'greader-get-attributes)
-    (define-key map (kbd "b")   #'greader-change-backend)
-    (define-key map (kbd "c") #'greader-compile-at-point)
-    map))
-
 (defvar greader-mode-map
   (let ((map (make-sparse-keymap)))
-    (define-key map (kbd greader-keymap-prefix) greader-prefix-keymap)
+    (define-key map (kbd "C-r s")   #'greader-toggle-tired-mode)
+    (define-key map (kbd "C-r r")   #'isearch-backward)
+    (define-key map (kbd "C-r SPC") #'greader-read)
+    (define-key map (kbd "C-r l")   #'greader-set-language)
+    (define-key map (kbd "C-r t")   #'greader-toggle-timer)
+    (define-key map (kbd "C-r f")   #'greader-get-attributes)
+    (define-key map (kbd "C-r b")   #'greader-change-backend)
+    (define-key map (kbd "C-r c") #'greader-compile-at-point)
     map))
 
 (defvar greader-reading-map
@@ -290,9 +275,6 @@ if set to t, when you call function `greader-read', that function sets a
     (define-key map (kbd "<left>")   #'greader-backward)
     (define-key map (kbd "<right>")   #'greader-forward)
     map))
-
-(eval-after-load 'greader-dict
-  '(define-key greader-prefix-keymap "d" greader-dict-prefix-map))
 (defvar greader-queue-mode)
 ;;;###autoload
 (define-minor-mode greader-mode
@@ -304,36 +286,12 @@ if set to t, when you call function `greader-read', that function sets a
     (when greader-queue-mode
       (greader-queue-mode -1))
     (greader-load-backends))))
-(defvar greader--current-buffer nil
-  "This variable is used by `greader-reading-mode'
-to know which buffer is currently being read.")
-
-(defun greader-current-buffer-p (&optional buffer)
-  "Return t if BUFFER is the current reading buffer, nil otherwise."
-  (unless buffer
-    (setq buffer (current-buffer)))
-  (equal buffer greader--current-buffer))
-
-(defun greader--set-current-buffer (&optional buffer)
-  "Set `greader--current-buffer' using BUFFER.
-If BUFFER is omitted or nil, use `current-buffer' as BUFFER."
-  (unless buffer
-    (setq buffer (current-buffer)))
-  (unless greader--current-buffer
-    (setq greader--current-buffer buffer)))
-
 ;;;###autoload
 (define-minor-mode greader-reading-mode
   nil
   :interactive nil
   :keymap greader-reading-map
-  :lighter " reading..."
-  (if greader-reading-mode
-      (progn
-	(setq greader--current-buffer (current-buffer))
-	(add-hook 'greader-before-get-sentence-hook #'greader--set-current-buffer))
-    (setq greader--current-buffer nil)
-    (remove-hook 'greader-before-get-sentence-hook #'greader--set-current-buffer)))
+  :lighter " reading...")
 
 (defun greader-set-bookmark-for-greader ()
   "Imposta il segnalibro ad ogni interruzione della lettura."
@@ -558,17 +516,32 @@ Optional argument EVENT ."
 (defun greader-build-args ()
   "Build the string that will be passed to the back-end."
   (greader-reset)
-  (let (args)
-    (push (greader-call-backend 'rate) args)
-    (when-let* ((lang (greader-call-backend 'lang)))
-      (push lang args))
-    (when-let* ((punc (greader-call-backend 'punctuation)))
-      (push punc args))
-    (when-let* ((extra (greader-call-backend 'extra)))
-      (unless (eq extra 'not-implemented)
-        (push extra args)))
-    (setq greader-backend (append greader-backend
-                                  (delete 'not-implemented (nreverse args))))))
+  (let (args arg)
+    (setq arg
+	  (greader-call-backend 'rate))
+    (setq args (append `(,arg) args))
+    (cond ((greader-call-backend 'lang)
+	   (setq arg
+		 (greader-call-backend 'lang))
+	   (setq args (append `(,arg) args))))
+    (cond ((greader-call-backend 'punctuation)
+	   (setq arg (greader-call-backend 'punctuation))
+	   (setq args (append `(,arg) args))))
+    (setq greader-backend (greader-call-backend 'executable))
+    (cond
+     (
+      (not
+       (eq
+	(greader-call-backend 'extra)
+	'not-implemented))
+      (setq arg (greader-call-backend 'extra))
+      (setq args (append `(,arg) args))))
+    (catch 'deleted
+      (dolist (argument args)
+	(when (equal argument 'not-implemented)
+	  (setq args (delete argument args))
+	  (throw 'deleted t))))
+    (setq greader-backend (append `(,greader-backend) args))))
 
 (defun greader-reset ()
   "Reset greader."
@@ -667,11 +640,7 @@ function, point jumps at the last position you called command `greader-read'."
 	  (greader-read-asynchronous ". end"))))))
 
 (defun greader-stop ()
-  "Stop reading of document.
-If `greader-dict-mode' and/or `greader-dict-toggle-filters' are
-active, the dictionary of pronunciation rules will be updated after
-calling all the hooks."
-
+  "Stops reading of document."
   (interactive)
   (cond
    ((and (> greader-elapsed-time 0) greader-timer-flag)
@@ -684,11 +653,7 @@ calling all the hooks."
     (setq-local greader-stop-timer 0)))
   (greader-set-greader-keymap)
   (greader-tts-stop)
-  (run-hooks 'greader-after-stop-hook)
-  (when (and (featurep 'greader-dict)
-             (or greader-dict-mode
-                 greader-dict-toggle-filters))
-    (greader-dict--update)))
+  (run-hooks 'greader-after-stop-hook))
 
 (defun greader-debug (arg)
   "Used to get some fast debugging.
@@ -1762,6 +1727,21 @@ this major mode to the variable `greader-continuous-modes'")))
 (defvar-local greader-study-start-position 1
   "The buffer position in which restart of reading will happen.")
 
+;;;###autoload
+(defun greader-study-set-position (pos)
+  "Set the position in which reading will restart.
+When called interactively, use the current position in the buffer.
+If `greader-study-mode' is not enabled, enable it first."
+  (interactive "d")
+  (unless greader-study-mode
+    (greader-study-mode 1))
+  (cond
+   ((not pos)
+    (user-error "Position must be a positive integer"))
+   ((< pos 1)
+    (user-error "position is not valid")))
+  (setq greader-study-start-position pos))
+
 ;; greader-study-restart is the function that will be added to the hook
 ;; `greader-before-finish-functions'.
 ;; The function simply returns to the beginning of the buffer or the
@@ -1786,21 +1766,6 @@ If `greader-region-mode' is enabled, restart will behave accordingly."
     (remove-hook 'greader-before-finish-functions
 		 #'greader-study-restart t)))
 
-;;;###autoload
-(defun greader-study-set-position (pos)
-  "Set the position in which reading will restart.
-When called interactively, use the current position in the buffer.
-If `greader-study-mode' is not enabled, enable it first."
-  (interactive "d")
-  (unless greader-study-mode
-    (greader-study-mode 1))
-  (cond
-   ((not pos)
-    (user-error "Position must be a positive integer"))
-   ((< pos 1)
-    (user-error "position is not valid")))
-  (setq greader-study-start-position pos))
-
 ;; Estimated reading time calculation.
 (defvar-local greader--buffer-words nil
   "Total buffer words.")
@@ -1894,14 +1859,5 @@ to the next sentence, or when you stop the reading."
 
 
 
-(defun greader-set-map-prefix (keys)
-  "Set `greader-keymap-prefix' to KEYS for the current session."
-  (interactive "kSet greader prefix to: ")
-  (let ((key-str (key-description keys)))
-    (define-key greader-mode-map (kbd greader-keymap-prefix) nil)
-    (setq greader-keymap-prefix key-str)
-    (define-key greader-mode-map (kbd greader-keymap-prefix) greader-prefix-keymap)
-    (message "greader prefix set to %s for the current session." key-str)))
-
 (provide 'greader)
 ;;; greader.el ends here
